//objectDemo_teacher by Andrew Rose
//measure IP traffic in terms of number of packets per IP/port/socket

//1.1. create a sockets object
//1.2. loop thru packets
//1.3. if IP packet, determine if you are source or destination, 
//1.4. print socket you are communicating with (IP address + port of other cpu)
//1.5. add to socket object (set new to 1, increment existing)
//1.7. sort keys of sockets object by highest number of packets, store as keys array
//1.8. output sorted key-value pairs
//1.9. calculate % totals
//challenge: filter by ip and port separately, add layer 2 data (mac address)

//Extension: bargraph

var host = "10.18.194.211"; //run ifconfig - put your IP here
var ipAddrs = {};
var ports = {};
var sockets = {}; // obj {socket: # packets}
var keys = []; //array of sockets - can be sorted by # packets

function setup() {
  createCanvas(400,500);
  createFileInput(gotFile); //user must select valid wireshark JSON
}

//called whenever a file is selected
function gotFile(file) {
  loadJSON(file.data, parseJSON);
}

//called once JSON is loaded
function parseJSON(packets) {
  //reset variables
  total = packets.length
  sockets = {}
  keys = []

  //look at each packet i in data
  for (var i = 0; i < packets.length; i++) {
    //create a layer variable
    var layer = packets[i]._source.layers; //['_source']['layers'];

    var ip = layer.ip;
    var other, port;
    //if there is an ip layer, identify if host is source or destination IP
    //determine protocol (TCP/UDP) and get the port used by other
    //use other's IP address and port number to create a socket
    //add socket to our inventory or incremenent if already exists
    if (ip) {
      if (ip["ip.src"] == host) {
        other = ip["ip.dst"]
        port = "dstport"
      } else {
        other = ip["ip.src"]
        port = "srcport"
      }
      //console.log("  IP: " + ip["ip.src"] + " --> " + ip["ip.dst"])
      var tcp = layer["tcp"];
      var socket;

      if (tcp) {
        socket = other + ": " + tcp["tcp." + port]
          //console.log("  TCP port: " + tcp["tcp.srcport"] + " --> " + tcp["tcp.dstport"])
      } else {
        var udp = layer["udp"];
        socket = other + ": " + udp["udp." + port]
          //console.log("  UDP port: " + +udp["udp.srcport"] + " --> " + udp["udp.dstport"])
      } //udp
    } //ip
    else { //layer 2 packet
      socket = "layer 2"
    }
    if (sockets[socket] === undefined) { // It's a new ip
      sockets[socket] = 1;
      keys.push(socket)
        // We've seen this word before!
    } else
      sockets[socket]++;
  } //for
  //sort keys (sockets) by greatest # packets
  keys.sort(function(a, b) {
    return (sockets[b] - sockets[a]);
  });
  //display results
  for (var i = 0; i < keys.length; i++) {
    console.log(keys[i] + ' (' + sockets[keys[i]] + ' packets-' +
      nf(sockets[keys[i]] / packets.length * 100, -1, 2) + '%)');
  }
  console.log(sockets)
  bargraph();
}

//extension
function bargraph() {
  background(255);
  //display top <j> IP addresses
  var j = 10;
  if (keys.length < 10)
    j = keys.length;
  for (var i = 0; i < j; i++) {
    //split IP address into subnets for color-coding
    var ipArr = split(keys[i], '.');
    if (ipArr.length > 3)
      fill(parseInt(ipArr[0]), parseInt(ipArr[1]), parseInt(ipArr[2]), 100); // parseInt(ipArr[3]));
    else
      fill(200);
    //caclulate percentage of total packets for each IP
    var perc = sockets[keys[i]] / total;
    //scale % to y-axis
    //var y = perc*height;
    var x = 3 * perc * width / 4 //- 100;
      //filter by percentage
    var spacing = 50;
    //draw rectangle along with IP address and statistics
    rect(0, i * spacing + 25, x, 20);
    fill(0);
    text(keys[i], 0, i * spacing + 20);
    fill(100);
    text(sockets[keys[i]], 5, i * spacing + 40)

    if (x < 10)
      text(nf(perc * 100, 2, 2) + "%", x + 30, i * spacing + 40);
    else
      text(nf(perc * 100, 2, 2) + "%", x + 10, i * spacing + 40);
  }
}